import { ComponentDrawInfo, ComponentConstructorInfo } from '../types';
import { ExtensionManager } from '../extensions';
import { SN } from './definitions';
export declare enum MUST_CALL_AND_RETURN_SUPER_METHOD {
    SUPER_APPLY = 0,
    SUPER_DRAW = 1,
    SUPER_DESTROY = 2,
    SUPER_CREATE_ELEMENT = 3,
    SUPER_HIDE = 4
}
/**
 * The data which be used in Extensions.
 * @property id - using in IdGenerator
 * @property boxElement - using in BoxElementGenerator
 * @property realLength - using in PositionCounter, existing in Axis only.
 * @property needed - using in ConflictFixer, existing in EventBody only.
 * @property space - using in ConflictFixer, existing in EventBody only.
 * */
export interface ExtraData {
    id?: string;
    boxElement?: HTMLElement;
    realLength?: number;
    needed?: {
        top: number;
        bottom: number;
    };
    space?: {
        top: number;
        bottom: number;
    };
}
export default abstract class Component {
    constructor({ ext, canvas, container }: ComponentConstructorInfo);
    /**
     * Theme name.
     * Be filled when the theme's class constructed.
     * */
    abstract theme: string;
    /**
     * Component name.
     * Be filled when the engine's class constructed.
     * */
    abstract name: SN;
    /**
     * An ext extra
     * @see ExtraData
     * */
    extraData: ExtraData;
    /**
     * The canvas where component draw on.
     * */
    canvas: HTMLCanvasElement;
    /**
     * The container which contain all of component's element.
     * */
    container: HTMLElement;
    /**
     * Manage all extensions & Help component communicate for each other.
     * */
    ext: ExtensionManager;
    /**
     * All info about draw. Except drawInfo, no more states be depended about draw.
     * The same drawInfo must has same drawn on canvas and container.
     * The this.gird should be merging in there in this.apply be called.
     * Must can unscathed passed this.drawInfo = JSON.parse(JSON.stringify(this.drawInfo)).
     * */
    abstract readonly drawInfo: ComponentDrawInfo;
    /** A ratio for draw, it's using when draw on High DPI display */
    ratio: number;
    /** Setup for High DPI */
    setupRadio(): void;
    /**
     * Optional. Be filled in this.createElement.
     * The HTML element which be needed by component draw.
     * */
    element?: HTMLElement;
    /**
     * Optional. Depend on the draw this component is need HTML element or not.
     * Create an HTML element base on this.drawInfo to fill this.element
     * and append to this.container.
     * Be called in this.apply and this.draw.
     * */
    createElement(): MUST_CALL_AND_RETURN_SUPER_METHOD;
    /**
     * Create a box object to fill this.drawInfo.box.
     * It should be called in apply() at least once.
     * Write this method if you need to change the behavior about box.
     * */
    abstract createBox(): void;
    /**
     * Means that the component has been destroyed.
     * Call any method on component when it has destroyed will got an error.
     * */
    destroyed: boolean;
    /**
     * Destroy self.
     * It should remove all element from dom and clear all trace in canvas if that's made by self.
     * */
    destroy(): MUST_CALL_AND_RETURN_SUPER_METHOD;
    /**
     * Apply this.drawInfo in component.
     * This method should't change any property in this.drawInfo except this.drawInfo.box.
     * @async
     * */
    apply(...args: any[]): Promise<MUST_CALL_AND_RETURN_SUPER_METHOD>;
    /**
     * Draw self base on this.drawInfo.
     * It should can be call multiple times.
     * Call this.apply before this.draw.
     * */
    draw(): MUST_CALL_AND_RETURN_SUPER_METHOD;
    /**
     * Hide all of trace created by self.
     * Hidden is not destroyed. Ensure the component is still available when this.hide have call.
     * This method will try set "visibility: 'hidden'" for this.element rather than remove it.
     * */
    hide(): MUST_CALL_AND_RETURN_SUPER_METHOD;
    /**
     * Set a whole drawInfo into component.
     * The component still need this.apply called before draw when this.importDrawInfo been called.
     * */
    importDrawInfo(drawInfo: ComponentDrawInfo): this;
    /**
     * Print log if the DEBUG is true.
     * @example this.l`Hello world`
     * */
    protected get l(): (stringArr: TemplateStringsArray, ...values: any[]) => void;
    /**
     * Verify a component is destroyed or not.
     * Throw an error if this.destroyed is true.
     * */
    protected checkDestroy(): void;
}
